"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WASI = void 0;
const preview1_1 = require("./preview1");
const util_1 = require("./util");
// eslint-disable-next-line spaced-comment
const kEmptyObject = /*#__PURE__*/ Object.freeze(/*#__PURE__*/ Object.create(null));
const kExitCode = Symbol('kExitCode');
const kSetMemory = Symbol('kSetMemory');
const kStarted = Symbol('kStarted');
const kInstance = Symbol('kInstance');
function setupInstance(self, instance) {
    (0, util_1.validateObject)(instance, 'instance');
    (0, util_1.validateObject)(instance.exports, 'instance.exports');
    self[kInstance] = instance;
    self[kSetMemory](instance.exports.memory);
}
// /** @public */
// export type WasiSnapshotPreview1 = Omit<_WASI, '_setMemory'>
function validateOptions(options) {
    var _a;
    (0, util_1.validateObject)(options, 'options');
    if (options.args !== undefined) {
        (0, util_1.validateArray)(options.args, 'options.args');
    }
    const args = ((_a = options.args) !== null && _a !== void 0 ? _a : []).map(String);
    const env = [];
    if (options.env !== undefined) {
        (0, util_1.validateObject)(options.env, 'options.env');
        Object.entries(options.env).forEach(({ 0: key, 1: value }) => {
            if (value !== undefined) {
                env.push(`${key}=${value}`);
            }
        });
    }
    const preopens = [];
    if (options.preopens !== undefined) {
        (0, util_1.validateObject)(options.preopens, 'options.preopens');
        Object.entries(options.preopens).forEach(({ 0: key, 1: value }) => preopens.push({ mappedPath: String(key), realPath: String(value) }));
    }
    if (preopens.length > 0) {
        if (options.fs === undefined) {
            throw new Error('filesystem is disabled, can not preopen directory');
        }
        try {
            (0, util_1.validateObject)(options.fs, 'options.fs');
        }
        catch (_) {
            throw new TypeError('Node.js fs like implementation is not provided');
        }
    }
    // if (options.filesystem !== undefined) {
    //   validateObject(options.filesystem, 'options.filesystem')
    //   validateString(options.filesystem.type, 'options.filesystem.type')
    //   if (options.filesystem.type !== 'memfs' && options.filesystem.type !== 'file-system-access-api') {
    //     throw new Error(`Filesystem type ${(options.filesystem as any).type as string} is not supported, only "memfs" and "file-system-access-api" is supported currently`)
    //   }
    //   try {
    //     validateObject(options.filesystem.fs, 'options.filesystem.fs')
    //   } catch (_) {
    //     throw new Error('Node.js fs like implementation is not provided')
    //   }
    // }
    if (options.print !== undefined)
        (0, util_1.validateFunction)(options.print, 'options.print');
    if (options.printErr !== undefined)
        (0, util_1.validateFunction)(options.printErr, 'options.printErr');
    if (options.returnOnExit !== undefined) {
        (0, util_1.validateBoolean)(options.returnOnExit, 'options.returnOnExit');
    }
    // const { stdin = 0, stdout = 1, stderr = 2 } = options
    // validateInt32(stdin, 'options.stdin', 0)
    // validateInt32(stdout, 'options.stdout', 0)
    // validateInt32(stderr, 'options.stderr', 0)
    // const stdio = [stdin, stdout, stderr] as const
    const stdio = [0, 1, 2];
    return {
        args,
        env,
        preopens,
        stdio
    };
}
/** @public */
class WASI {
    constructor(setMemory, wrap) {
        this[kSetMemory] = setMemory;
        this.wasiImport = wrap;
        this[kStarted] = false;
        this[kExitCode] = 0;
        this[kInstance] = undefined;
    }
    static createSync(options = kEmptyObject) {
        const { args, env, preopens, stdio } = validateOptions(options);
        const wrap = preview1_1.WASI.createSync(args, env, preopens, stdio, options.fs, options.print, options.printErr);
        const setMemory = wrap._setMemory;
        delete wrap._setMemory;
        const _this = new WASI(setMemory, wrap);
        if (options.returnOnExit) {
            wrap.proc_exit = wasiReturnOnProcExit.bind(_this);
        }
        return _this;
    }
    static async createAsync(options = kEmptyObject) {
        const { args, env, preopens, stdio } = validateOptions(options);
        if (options.asyncify !== undefined) {
            (0, util_1.validateObject)(options.asyncify, 'options.asyncify');
            (0, util_1.validateFunction)(options.asyncify.wrapImportFunction, 'options.asyncify.wrapImportFunction');
        }
        const wrap = await preview1_1.WASI.createAsync(args, env, preopens, stdio, options.fs, options.print, options.printErr, options.asyncify);
        const setMemory = wrap._setMemory;
        delete wrap._setMemory;
        const _this = new WASI(setMemory, wrap);
        if (options.returnOnExit) {
            wrap.proc_exit = wasiReturnOnProcExit.bind(_this);
        }
        return _this;
    }
    // Must not export _initialize, must export _start
    start(instance) {
        if (this[kStarted]) {
            throw new Error('WASI instance has already started');
        }
        this[kStarted] = true;
        setupInstance(this, instance);
        const { _start, _initialize } = this[kInstance].exports;
        (0, util_1.validateFunction)(_start, 'instance.exports._start');
        (0, util_1.validateUndefined)(_initialize, 'instance.exports._initialize');
        let ret;
        try {
            ret = _start();
        }
        catch (err) {
            if (err !== kExitCode) {
                throw err;
            }
        }
        if (ret instanceof Promise) {
            return ret.then(() => this[kExitCode], (err) => {
                if (err !== kExitCode) {
                    throw err;
                }
                return this[kExitCode];
            });
        }
        return this[kExitCode];
    }
    // Must not export _start, may optionally export _initialize
    initialize(instance) {
        if (this[kStarted]) {
            throw new Error('WASI instance has already started');
        }
        this[kStarted] = true;
        setupInstance(this, instance);
        const { _start, _initialize } = this[kInstance].exports;
        (0, util_1.validateUndefined)(_start, 'instance.exports._start');
        if (_initialize !== undefined) {
            (0, util_1.validateFunction)(_initialize, 'instance.exports._initialize');
            return _initialize();
        }
    }
}
exports.WASI = WASI;
function wasiReturnOnProcExit(rval) {
    this[kExitCode] = rval;
    // eslint-disable-next-line @typescript-eslint/no-throw-literal
    throw kExitCode;
}
//# sourceMappingURL=index.js.map