import { _WebAssembly } from "./webassembly.mjs";
import { Asyncify } from "./asyncify.mjs";
function validateImports(imports) {
    if (imports && typeof imports !== 'object') {
        throw new TypeError('imports must be an object or undefined');
    }
}
async function fetchWasm(urlOrBuffer, imports) {
    if (typeof wx !== 'undefined' && typeof __wxConfig !== 'undefined') {
        return await _WebAssembly.instantiate(urlOrBuffer, imports);
    }
    const response = await fetch(urlOrBuffer);
    const buffer = await response.arrayBuffer();
    const source = await _WebAssembly.instantiate(buffer, imports);
    return source;
}
/** @public */
export async function load(urlOrBuffer, imports) {
    validateImports(imports);
    imports = imports !== null && imports !== void 0 ? imports : {};
    let source;
    if (urlOrBuffer instanceof ArrayBuffer || ArrayBuffer.isView(urlOrBuffer)) {
        source = await _WebAssembly.instantiate(urlOrBuffer, imports);
        return source;
    }
    if (typeof urlOrBuffer !== 'string' && !(urlOrBuffer instanceof URL)) {
        throw new TypeError('Invalid source');
    }
    if (typeof _WebAssembly.instantiateStreaming === 'function') {
        try {
            source = await _WebAssembly.instantiateStreaming(fetch(urlOrBuffer), imports);
        }
        catch (_) {
            source = await fetchWasm(urlOrBuffer, imports);
        }
    }
    else {
        source = await fetchWasm(urlOrBuffer, imports);
    }
    return source;
}
/** @public */
export async function asyncifyLoad(asyncify, urlOrBuffer, imports) {
    var _a;
    validateImports(imports);
    imports = imports !== null && imports !== void 0 ? imports : {};
    const asyncifyHelper = new Asyncify();
    imports = asyncifyHelper.wrapImports(imports);
    const source = await load(urlOrBuffer, imports);
    const memory = source.instance.exports.memory || ((_a = imports.env) === null || _a === void 0 ? void 0 : _a.memory);
    return { module: source.module, instance: asyncifyHelper.init(memory, source.instance, asyncify) };
}
/** @public */
export function loadSync(buffer, imports) {
    if ((buffer instanceof ArrayBuffer) && !ArrayBuffer.isView(buffer)) {
        throw new TypeError('Invalid source');
    }
    validateImports(imports);
    imports = imports !== null && imports !== void 0 ? imports : {};
    const module = new _WebAssembly.Module(buffer);
    const instance = new _WebAssembly.Instance(module, imports);
    const source = { instance, module };
    return source;
}
/** @public */
export function asyncifyLoadSync(asyncify, buffer, imports) {
    var _a;
    validateImports(imports);
    imports = imports !== null && imports !== void 0 ? imports : {};
    const asyncifyHelper = new Asyncify();
    imports = asyncifyHelper.wrapImports(imports);
    const source = loadSync(buffer, imports);
    const memory = source.instance.exports.memory || ((_a = imports.env) === null || _a === void 0 ? void 0 : _a.memory);
    return { module: source.module, instance: asyncifyHelper.init(memory, source.instance, asyncify) };
}
